// Importing necessary functions from the d3 library
import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot'; // Importing custom scatter plot function

// URL of the CSV file containing wildlife data
const csvUrl =
  'https://gist.githubusercontent.com/nandinimaddula/f8d0f9b91d0677ea69149d66545ff462/raw/4be7c68d466a8bbf3cf08f21c5c4f7f84558c03c/Wildlife.csv';

// Parsing function to convert string values to numbers
const parseRow = (d) => {
  // Parsing population density to number
  d.population_density = +d.population_density;
  // Parsing altitude meters to number
  d.altitude_meters = +d.altitude_meters;
  // Parsing weight in kg to number
  d.weight_kg = +d.weight_kg;
  // Parsing temperature in Celsius to number
  d.temperature_celsius = +d.temperature_celsius;
  return d;
};

// Determine the width and height of the SVG based on the window size
const width = window.innerWidth;
const height = window.innerHeight;

// Append an SVG element to the body
const svg = select('body')
  // Appending an SVG element
  .append('svg')
  // Setting the width of SVG
  .attr('width', width)
  // Setting the height of SVG
  .attr('height', height);

// Main function to create the scatter plot
const main = async () => {
  // Create a scatter plot instance
  const plot = scatterPlot()
    // Setting the width of the plot
    .width(width)
    // Setting the height of the plot
    .height(height)
    // Load data from the CSV file and apply the parsing function
    .data(await csv(csvUrl, parseRow))
    // Adjust the X-axis value to population density
    .xValue((d) => d.population_density)
    // Adjust the Y-axis value to altitude in meters
    .yValue((d) => d.altitude_meters)
    // Adjust the radius X-value to weight in kg
    .rxValue((d) => d.weight_kg)
    // Adjust the radius Y-value to temperature in Celsius
    .ryValue((d) => d.temperature_celsius)
    // Assign the animal type value to determine symbol and color
    .animal_typeValue((d) => d.animal_type)
    // Assign the animal type value to determine symbol and color
    .symbolValue((d) => d.animal_type)
    // Set margins for the plot
    .margin({
      top: 20,
      right: 20,
      bottom: 40,
      left: 60,
    })
    // Set the radius for the data points
    .radius(5);

  // Call the plot function on the SVG element
  svg.call(plot);

  // Update the scatterplot data after a specific time interval
  let i = 0;
  setInterval(() => {
    // Define an array of column names for cyclic update
    const columns = [
      'population_density',
      'altitude_meters',
      'weight_kg',
      'temperature_celsius',
    ];
    // Update the X-axis value based on the current column
    plot.xValue((d) => d[columns[i % columns.length]]);
    // Call the plot function again to update the plot
    svg.call(plot);
    // Increment the index for the next column update
    i++;
  }, 1000);
};

// Call the main function to execute the plot
main();
